@@ -1,9 +1,9 @@
 import asyncio
-import logging
 from functools import wraps
-from typing import List, Optional, SupportsInt  # noqa
+from typing import Any, Callable, List, Optional, SupportsInt, Tuple, TypeVar, Union
 
 import asyncpg
+from asyncpg.transaction import Transaction
 from pypika import PostgreSQLQuery
 
 from tortoise.backends.asyncpg.executor import AsyncpgExecutor
@@ -13,6 +13,10 @@ from tortoise.backends.base.client import (
     BaseTransactionWrapper,
     Capabilities,
     ConnectionWrapper,
+    NestedTransactionPooledContext,
+    PoolConnectionWrapper,
+    TransactionContext,
+    TransactionContextPooled,
 )
 from tortoise.exceptions import (
     DBConnectionError,
@@ -20,42 +24,12 @@ from tortoise.exceptions import (
     OperationalError,
     TransactionManagementError,
 )
-from tortoise.transactions import current_transaction_map
+
+FuncType = Callable[..., Any]
+F = TypeVar("F", bound=FuncType)
 
 
-def retry_connection(func):
-    @wraps(func)
-    async def retry_connection_(self, *args):
-        try:
-            return await func(self, *args)
-        except (
-            asyncpg.PostgresConnectionError,
-            asyncpg.ConnectionDoesNotExistError,
-            asyncpg.ConnectionFailureError,
-            asyncpg.InterfaceError,
-        ):
-            # Here we assume that a connection error has happened
-            # Re-create connection and re-try the function call once only.
-            if getattr(self, "transaction", None):
-                self._finalized = True
-                raise TransactionManagementError("Connection gone away during transaction")
-            await self._lock.acquire()
-            logging.info("Attempting reconnect")
-            try:
-                await self._close()
-                await self.create_connection(with_db=True)
-                logging.info("Reconnected")
-            except Exception as e:
-                raise DBConnectionError("Failed to reconnect: %s", str(e))
-            finally:
-                self._lock.release()
-
-            return await func(self, *args)
-
-    return retry_connection_
-
-
-def translate_exceptions(func):
+def translate_exceptions(func: F) -> F:
     @wraps(func)
     async def translate_exceptions_(self, *args):
         try:
@@ -64,8 +38,10 @@ def translate_exceptions(func):
             raise OperationalError(exc)
         except asyncpg.IntegrityConstraintViolationError as exc:
             raise IntegrityError(exc)
+        except asyncpg.InvalidTransactionStateError as exc:  # pragma: nocoverage
+            raise TransactionManagementError(exc)
 
-    return translate_exceptions_
+    return translate_exceptions_  # type: ignore
 
 
 class AsyncpgDBClient(BaseDBAsyncClient):
@@ -76,7 +52,7 @@ class AsyncpgDBClient(BaseDBAsyncClient):
     capabilities = Capabilities("postgres")
 
     def __init__(
-        self, user: str, password: str, database: str, host: str, port: SupportsInt, **kwargs
+        self, user: str, password: str, database: str, host: str, port: SupportsInt, **kwargs: Any
     ) -> None:
         super().__init__(**kwargs)
 
@@ -91,14 +67,12 @@ class AsyncpgDBClient(BaseDBAsyncClient):
         self.extra.pop("fetch_inserted", None)
         self.extra.pop("loop", None)
         self.extra.pop("connection_class", None)
+        self.pool_minsize = int(self.extra.pop("minsize", 1))
+        self.pool_maxsize = int(self.extra.pop("maxsize", 5))
 
-        self._template = {}  # type: dict
-        self._connection = None  # Type: Optional[asyncpg.Connection]
-        self._lock = asyncio.Lock()
-
-        self._transaction_class = type(
-            "TransactionWrapper", (TransactionWrapper, self.__class__), {}
-        )
+        self._template: dict = {}
+        self._pool: Optional[asyncpg.pool] = None
+        self._connection = None
 
     async def create_connection(self, with_db: bool) -> None:
         self._template = {
@@ -106,78 +80,108 @@ class AsyncpgDBClient(BaseDBAsyncClient):
             "port": self.port,
             "user": self.user,
             "database": self.database if with_db else None,
+            "min_size": self.pool_minsize,
+            "max_size": self.pool_maxsize,
             **self.extra,
         }
-        try:
-            self._connection = await asyncpg.connect(None, password=self.password, **self._template)
-            self.log.debug(
-                "Created connection %s with params: %s", self._connection, self._template
-            )
-        except asyncpg.InvalidCatalogNameError:
-            raise DBConnectionError(
-                "Can't establish connection to database {}".format(self.database)
-            )
-        # Set post-connection variables
         if self.schema:
-            await self.execute_script("SET search_path TO {}".format(self.schema))
+            self._template["server_settings"] = {"search_path": self.schema}
+        try:
+            self._pool = await asyncpg.create_pool(None, password=self.password, **self._template)
+            self.log.debug("Created connection pool %s with params: %s", self._pool, self._template)
+        except asyncpg.InvalidCatalogNameError:
+            raise DBConnectionError(f"Can't establish connection to database {self.database}")
+        # Set post-connection variables
+
+    async def _expire_connections(self) -> None:
+        if self._pool:  # pragma: nobranch
+            await self._pool.expire_connections()
 
     async def _close(self) -> None:
-        if self._connection:  # pragma: nobranch
-            await self._connection.close()
-            self.log.debug("Closed connection %s with params: %s", self._connection, self._template)
-            self._template.clear()
+        if self._pool:  # pragma: nobranch
+            try:
+                await asyncio.wait_for(self._pool.close(), 10)
+            except asyncio.TimeoutError:  # pragma: nocoverage
+                self._pool.terminate()
+            self._pool = None
+            self.log.debug("Closed connection pool %s with params: %s", self._pool, self._template)
 
     async def close(self) -> None:
         await self._close()
-        self._connection = None
+        self._template.clear()
 
     async def db_create(self) -> None:
         await self.create_connection(with_db=False)
-        await self.execute_script(
-            'CREATE DATABASE "{}" OWNER "{}"'.format(self.database, self.user)
-        )
+        await self.execute_script(f'CREATE DATABASE "{self.database}" OWNER "{self.user}"')
         await self.close()
 
     async def db_delete(self) -> None:
         await self.create_connection(with_db=False)
         try:
-            await self.execute_script('DROP DATABASE "{}"'.format(self.database))
+            await self.execute_script(f'DROP DATABASE "{self.database}"')
         except asyncpg.InvalidCatalogNameError:  # pragma: nocoverage
             pass
         await self.close()
 
-    def acquire_connection(self) -> ConnectionWrapper:
-        return ConnectionWrapper(self._connection, self._lock)
+    def acquire_connection(self) -> Union["PoolConnectionWrapper", "ConnectionWrapper"]:
+        return PoolConnectionWrapper(self._pool)
 
-    def _in_transaction(self) -> "TransactionWrapper":
-        return self._transaction_class(self)
+    def _in_transaction(self) -> "TransactionContext":
+        return TransactionContextPooled(TransactionWrapper(self))
 
     @translate_exceptions
-    @retry_connection
     async def execute_insert(self, query: str, values: list) -> Optional[asyncpg.Record]:
         async with self.acquire_connection() as connection:
             self.log.debug("%s: %s", query, values)
             # TODO: Cache prepared statement
-            stmt = await connection.prepare(query)
-            return await stmt.fetchrow(*values)
+            return await connection.fetchrow(query, *values)
 
     @translate_exceptions
-    @retry_connection
     async def execute_many(self, query: str, values: list) -> None:
         async with self.acquire_connection() as connection:
             self.log.debug("%s: %s", query, values)
             # TODO: Consider using copy_records_to_table instead
-            await connection.executemany(query, values)
+            transaction = connection.transaction()
+            await transaction.start()
+            try:
+                await connection.executemany(query, values)
+            except Exception:
+                await transaction.rollback()
+                raise
+            else:
+                await transaction.commit()
 
     @translate_exceptions
-    @retry_connection
-    async def execute_query(self, query: str) -> List[dict]:
+    async def execute_query(
+        self, query: str, values: Optional[list] = None
+    ) -> Tuple[int, List[dict]]:
         async with self.acquire_connection() as connection:
-            self.log.debug(query)
-            return await connection.fetch(query)
+            self.log.debug("%s: %s", query, values)
+            if values:
+                params = [query, *values]
+            else:
+                params = [query]
+            if query.startswith("UPDATE") or query.startswith("DELETE"):
+                res = await connection.execute(*params)
+                try:
+                    rows_affected = int(res.split(" ")[1])
+                except Exception:  # pragma: nocoverage
+                    rows_affected = 0
+                return rows_affected, []
+            else:
+                rows = await connection.fetch(*params)
+                return len(rows), rows
+
+    @translate_exceptions
+    async def execute_query_dict(self, query: str, values: Optional[list] = None) -> List[dict]:
+        async with self.acquire_connection() as connection:
+            self.log.debug("%s: %s", query, values)
+            if values:
+                # TODO: Cache prepared statement
+                return list(map(dict, await connection.fetch(query, *values)))
+            return list(map(dict, await connection.fetch(query)))
 
     @translate_exceptions
-    @retry_connection
     async def execute_script(self, query: str) -> None:
         async with self.acquire_connection() as connection:
             self.log.debug(query)
@@ -185,45 +189,42 @@ class AsyncpgDBClient(BaseDBAsyncClient):
 
 
 class TransactionWrapper(AsyncpgDBClient, BaseTransactionWrapper):
-    def __init__(self, connection) -> None:
-        self._connection = connection._connection
-        self._lock = connection._lock
-        self.log = logging.getLogger("db_client")
-        self._transaction_class = self.__class__
-        self._old_context_value = None
+    def __init__(self, connection: AsyncpgDBClient) -> None:
+        self._connection: asyncpg.Connection = connection._connection
+        self._lock = asyncio.Lock()
+        self._trxlock = asyncio.Lock()
+        self.log = connection.log
         self.connection_name = connection.connection_name
-        self.transaction = None
+        self.transaction: Transaction = None
         self._finalized = False
         self._parent = connection
 
-    async def create_connection(self, with_db: bool) -> None:
-        await self._parent.create_connection(with_db)
-        self._connection = self._parent._connection
+    def _in_transaction(self) -> "TransactionContext":
+        return NestedTransactionPooledContext(self)
 
-    async def _close(self) -> None:
-        await self._parent._close()
-        self._connection = self._parent._connection
+    def acquire_connection(self) -> "ConnectionWrapper":
+        return ConnectionWrapper(self._connection, self._lock)
 
-    @retry_connection
-    async def start(self):
+    @translate_exceptions
+    async def execute_many(self, query: str, values: list) -> None:
+        async with self.acquire_connection() as connection:
+            self.log.debug("%s: %s", query, values)
+            # TODO: Consider using copy_records_to_table instead
+            await connection.executemany(query, values)
+
+    @translate_exceptions
+    async def start(self) -> None:
         self.transaction = self._connection.transaction()
         await self.transaction.start()
-        current_transaction = current_transaction_map[self.connection_name]
-        self._old_context_value = current_transaction.get()
-        current_transaction.set(self)
 
-    def release(self) -> None:
-        self._finalized = True
-        current_transaction_map[self.connection_name].set(self._old_context_value)
-
-    async def commit(self):
+    async def commit(self) -> None:
         if self._finalized:
             raise TransactionManagementError("Transaction already finalised")
         await self.transaction.commit()
-        self.release()
+        self._finalized = True
 
-    async def rollback(self):
+    async def rollback(self) -> None:
         if self._finalized:
             raise TransactionManagementError("Transaction already finalised")
         await self.transaction.rollback()
-        self.release()
+        self._finalized = True
