@@ -1,13 +1,13 @@
 import urllib.parse as urlparse
 import uuid
-from typing import Any, Dict, List, Optional  # noqa
+from typing import Any, Dict, List, Optional
 
 from tortoise.exceptions import ConfigurationError
 
 urlparse.uses_netloc.append("postgres")
 urlparse.uses_netloc.append("sqlite")
 urlparse.uses_netloc.append("mysql")
-DB_LOOKUP = {
+DB_LOOKUP: Dict[str, Dict[str, Any]] = {
     "postgres": {
         "engine": "tortoise.backends.asyncpg",
         "vmap": {
@@ -46,7 +46,7 @@ DB_LOOKUP = {
             "username": "user",
             "password": "password",
         },
-        "defaults": {"port": 3306, "charset": "utf8mb4"},
+        "defaults": {"port": 3306, "charset": "utf8mb4", "sql_mode": "STRICT_TRANS_TABLES"},
         "cast": {
             "minsize": int,
             "maxsize": int,
@@ -56,18 +56,18 @@ DB_LOOKUP = {
             "use_unicode": bool,
         },
     },
-}  # type: Dict[str, Dict[str, Any]]
+}
 
 
 def expand_db_url(db_url: str, testing: bool = False) -> dict:
     url = urlparse.urlparse(db_url)
     if url.scheme not in DB_LOOKUP:
-        raise ConfigurationError("Unknown DB scheme: {}".format(url.scheme))
+        raise ConfigurationError(f"Unknown DB scheme: {url.scheme}")
 
     db_backend = url.scheme
     db = DB_LOOKUP[db_backend]
     if db.get("skip_first_char", True):
-        path = url.path[1:]  # type: Optional[str]
+        path: Optional[str] = url.path[1:]
     else:
         path = url.netloc + url.path
 
@@ -77,7 +77,7 @@ def expand_db_url(db_url: str, testing: bool = False) -> dict:
         # Other database backend accepts database name being None (but not empty string).
         path = None
 
-    params = {}  # type: dict
+    params: dict = {}
     for key, val in db["defaults"].items():
         params[key] = val
     for key, val in urlparse.parse_qs(url.query).items():
@@ -88,7 +88,7 @@ def expand_db_url(db_url: str, testing: bool = False) -> dict:
         path = path.replace("\\{", "{").replace("\\}", "}")
         path = path.format(uuid.uuid4().hex)
 
-    vmap = {}  # type: dict
+    vmap: dict = {}
     vmap.update(db["vmap"])
     params[vmap["path"]] = path
     if vmap.get("hostname"):
