@@ -1,9 +1,11 @@
 import asyncio
 import os as _os
-from asyncio.selector_events import BaseSelectorEventLoop
+import unittest
+from asyncio.events import AbstractEventLoop
 from functools import wraps
-from typing import Any, List, Optional
-from unittest import SkipTest, expectedFailure, skip, skipIf, skipUnless  # noqa
+from typing import Any, Callable, List, Optional
+from unittest import SkipTest, expectedFailure, skip, skipIf, skipUnless
+from unittest.result import TestResult
 
 from asynctest import TestCase as _TestCase
 from asynctest import _fail_on
@@ -12,7 +14,7 @@ from asynctest.case import _Policy
 from tortoise import Tortoise
 from tortoise.backends.base.config_generator import generate_config as _generate_config
 from tortoise.exceptions import DBConnectionError
-from tortoise.transactions import current_transaction_map, start_transaction
+from tortoise.transactions import current_transaction_map
 
 __all__ = (
     "SimpleTestCase",
@@ -31,6 +33,7 @@ __all__ = (
     "skipUnless",
 )
 _TORTOISE_TEST_DB = "sqlite://:memory:"
+# pylint: disable=W0201
 
 expectedFailure.__doc__ = """
 Mark test as expecting failiure.
@@ -38,12 +41,12 @@ Mark test as expecting failiure.
 On success it will be marked as unexpected success.
 """
 
-_CONFIG = {}  # type: dict
-_CONNECTIONS = {}  # type: dict
+_CONFIG: dict = {}
+_CONNECTIONS: dict = {}
 _SELECTOR = None
-_LOOP = None  # type: BaseSelectorEventLoop
-_MODULES = []  # type: List[str]
-_CONN_MAP = {}  # type: dict
+_LOOP: AbstractEventLoop = None  # type: ignore
+_MODULES: List[str] = []
+_CONN_MAP: dict = {}
 
 
 def getDBConfig(app_label: str, modules: List[str]) -> dict:
@@ -81,7 +84,7 @@ def _restore_default() -> None:
 
 
 def initializer(
-    modules: List[str], db_url: Optional[str] = None, loop: Optional[BaseSelectorEventLoop] = None
+    modules: List[str], db_url: Optional[str] = None, loop: Optional[AbstractEventLoop] = None
 ) -> None:
     """
     Sets up the DB for testing. Must be called as part of test environment setup.
@@ -120,11 +123,11 @@ def finalizer() -> None:
     """
     _restore_default()
     loop = _LOOP
-    loop._selector = _SELECTOR
+    loop._selector = _SELECTOR  # type: ignore
     loop.run_until_complete(Tortoise._drop_databases())
 
 
-def env_initializer() -> None:
+def env_initializer() -> None:  # pragma: nocoverage
     """
     Calls ``initializer()`` with parameters mapped from environment variables.
 
@@ -133,7 +136,7 @@ def env_initializer() -> None:
     ``TORTOISE_TEST_DB``:
         The db_url of the test db. *(optional*)
     """
-    modules = str(_os.environ.get("TORTOISE_TEST_MODULES", "tortoise.tests.testmodels")).split(",")
+    modules = str(_os.environ.get("TORTOISE_TEST_MODULES", "tests.testmodels")).split(",")
     db_url = _os.environ.get("TORTOISE_TEST_DB", "sqlite://:memory:")
     if not modules:  # pragma: nocoverage
         raise Exception("TORTOISE_TEST_MODULES envvar not defined")
@@ -158,7 +161,6 @@ class SimpleTestCase(_TestCase):  # type: ignore
     use_default_loop = True
 
     def _init_loop(self) -> None:
-        # pylint: disable=W0201
         if self.use_default_loop:
             self.loop = _LOOP
             loop = None
@@ -177,30 +179,29 @@ class SimpleTestCase(_TestCase):  # type: ignore
     async def _tearDownDB(self) -> None:
         pass
 
-    def _setUp(self) -> None:
-        self._init_loop()
+    async def _setUp(self) -> None:
 
         # initialize post-test checks
         test = getattr(self, self._testMethodName)
         checker = getattr(test, _fail_on._FAIL_ON_ATTR, None)
-        self._checker = checker or _fail_on._fail_on()  # pylint: disable=W0201
+        self._checker = checker or _fail_on._fail_on()
         self._checker.before_test(self)
 
-        self.loop.run_until_complete(self._setUpDB())
+        await self._setUpDB()
         if asyncio.iscoroutinefunction(self.setUp):
-            self.loop.run_until_complete(self.setUp())
+            await self.setUp()
         else:
             self.setUp()
 
         # don't take into account if the loop ran during setUp
-        self.loop._asynctest_ran = False
+        self.loop._asynctest_ran = False  # type: ignore
 
-    def _tearDown(self) -> None:
+    async def _tearDown(self) -> None:
         if asyncio.iscoroutinefunction(self.tearDown):
-            self.loop.run_until_complete(self.tearDown())
+            await self.tearDown()
         else:
             self.tearDown()
-        self.loop.run_until_complete(self._tearDownDB())
+        await self._tearDownDB()
         Tortoise.apps = {}
         Tortoise._connections = {}
         Tortoise._inited = False
@@ -208,6 +209,87 @@ class SimpleTestCase(_TestCase):  # type: ignore
         # post-test checks
         self._checker.check_test(self)
 
+    # Override unittest.TestCase methods which call setUp() and tearDown()
+    def run(self, result: Optional[TestResult] = None) -> Optional[TestResult]:
+        orig_result = result
+        if result is None:  # pragma: nocoverage
+            result = self.defaultTestResult()
+            startTestRun = getattr(result, "startTestRun", None)
+            if startTestRun is not None:
+                startTestRun()
+
+        result.startTest(self)
+
+        testMethod = getattr(self, self._testMethodName)
+        if getattr(self.__class__, "__unittest_skip__", False) or getattr(
+            testMethod, "__unittest_skip__", False
+        ):
+            # If the class or method was skipped.
+            try:
+                skip_why = getattr(self.__class__, "__unittest_skip_why__", "") or getattr(
+                    testMethod, "__unittest_skip_why__", ""
+                )
+                self._addSkip(result, self, skip_why)
+            finally:
+                result.stopTest(self)
+            return None
+        expecting_failure = getattr(testMethod, "__unittest_expecting_failure__", False)
+        outcome = unittest.case._Outcome(result)  # type: ignore
+        try:
+            self._outcome = outcome
+
+            self._init_loop()
+
+            self.loop.run_until_complete(self._run_outcome(outcome, expecting_failure, testMethod))
+
+            self.loop.run_until_complete(self.doCleanups())
+            self._unset_loop()
+            for test, reason in outcome.skipped:
+                self._addSkip(result, test, reason)
+            self._feedErrorsToResult(result, outcome.errors)
+            if outcome.success:
+                if expecting_failure:
+                    if outcome.expectedFailure:
+                        self._addExpectedFailure(result, outcome.expectedFailure)
+                    else:  # pragma: nocoverage
+                        self._addUnexpectedSuccess(result)
+                else:
+                    result.addSuccess(self)
+            return result
+        finally:
+            result.stopTest(self)
+            if orig_result is None:  # pragma: nocoverage
+                stopTestRun = getattr(result, "stopTestRun", None)
+                if stopTestRun is not None:
+                    stopTestRun()  # pylint: disable=E1102
+
+            # explicitly break reference cycles:
+            # outcome.errors -> frame -> outcome -> outcome.errors
+            # outcome.expectedFailure -> frame -> outcome -> outcome.expectedFailure
+            outcome.errors.clear()
+            outcome.expectedFailure = None
+
+            # clear the outcome, no more needed
+            self._outcome = None
+
+    async def _run_outcome(self, outcome, expecting_failure: bool, testMethod: Callable) -> None:
+        with outcome.testPartExecutor(self):
+            await self._setUp()
+        if outcome.success:
+            outcome.expecting_failure = expecting_failure
+            with outcome.testPartExecutor(self, isTest=True):
+                await self._run_test_method(testMethod)
+            outcome.expecting_failure = False
+            with outcome.testPartExecutor(self):
+                await self._tearDown()
+
+    async def _run_test_method(self, method: Callable) -> None:
+        # If the method is a coroutine or returns a coroutine, run it on the
+        # loop
+        result = method()
+        if asyncio.iscoroutine(result):
+            await result
+
 
 class IsolatedTestCase(SimpleTestCase):
     """
@@ -221,11 +303,14 @@ class IsolatedTestCase(SimpleTestCase):
 
     It will create and destroy a new DB instance for every test.
     This is obviously slow, but guarantees a fresh DB.
+
+    If you define a ``tortoise_test_modules`` list, it overrides the DB setup module for the tests.
     """
 
-    # pylint: disable=C0103,W0201
+    tortoise_test_modules: List[str] = []
+
     async def _setUpDB(self) -> None:
-        config = getDBConfig(app_label="models", modules=_MODULES)
+        config = getDBConfig(app_label="models", modules=self.tortoise_test_modules or _MODULES)
         await Tortoise.init(config, _create_db=True)
         await Tortoise.generate_schemas(safe=False)
         self._connections = Tortoise._connections.copy()
@@ -245,21 +330,40 @@ class TruncationTestCase(SimpleTestCase):
     Note that usage of this does not guarantee that auto-number-pks will be reset to 1.
     """
 
-    # pylint: disable=C0103,W0201
     async def _setUpDB(self) -> None:
         _restore_default()
 
     async def _tearDownDB(self) -> None:
         _restore_default()
-        # TODO: This is a naïve implementation: Will fail to clear M2M and non-cascade foreign keys
+        # TODO: This is a naive implementation: Will fail to clear M2M and non-cascade foreign keys
         for app in Tortoise.apps.values():
             for model in app.values():
-                await model._meta.db.execute_script(
-                    "DELETE FROM {}".format(model._meta.table)  # nosec
-                )
+                await model._meta.db.execute_script(f"DELETE FROM {model._meta.table}")  # nosec
 
 
-class TestCase(SimpleTestCase):
+class TransactionTestContext:
+    __slots__ = ("connection", "connection_name", "token")
+
+    def __init__(self, connection) -> None:
+        self.connection = connection
+        self.connection_name = connection.connection_name
+
+    async def __aenter__(self):
+        current_transaction = current_transaction_map[self.connection_name]
+        self.token = current_transaction.set(self.connection)
+        if hasattr(self.connection, "_parent"):
+            self.connection._connection = await self.connection._parent._pool.acquire()
+        await self.connection.start()
+        return self.connection
+
+    async def __aexit__(self, exc_type, exc_val, exc_tb) -> None:
+        await self.connection.rollback()
+        if hasattr(self.connection, "_parent"):
+            await self.connection._parent._pool.release(self.connection._connection)
+        current_transaction_map[self.connection_name].reset(self.token)
+
+
+class TestCase(TruncationTestCase):
     """
     An asyncio capable test class that will ensure that each test will be run at
     separate transaction that will rollback on finish.
@@ -267,13 +371,24 @@ class TestCase(SimpleTestCase):
     This is a fast test runner. Don't use it if your test uses transactions.
     """
 
-    async def _setUpDB(self) -> None:
+    async def _run_outcome(self, outcome, expecting_failure, testMethod) -> None:
         _restore_default()
-        self.transaction = await start_transaction()  # pylint: disable=W0201
+        self.__db__ = Tortoise.get_connection("models")
+        if self.__db__.capabilities.supports_transactions:
+            connection = self.__db__._in_transaction().connection
+            async with TransactionTestContext(connection):
+                await super()._run_outcome(outcome, expecting_failure, testMethod)
+        else:
+            await super()._run_outcome(outcome, expecting_failure, testMethod)
+
+    async def _setUpDB(self) -> None:
+        pass
 
     async def _tearDownDB(self) -> None:
-        _restore_default()
-        await self.transaction.rollback()
+        if self.__db__.capabilities.supports_transactions:
+            _restore_default()
+        else:
+            await super()._tearDownDB()
 
 
 def requireCapability(connection_name: str = "models", **conditions: Any):
@@ -291,19 +406,44 @@ def requireCapability(connection_name: str = "models", **conditions: Any):
         async def test_run_sqlite_only(self):
             ...
 
+    Or to conditionally skip a class:
+
+    .. code-block:: python3
+
+        @requireCapability(dialect='sqlite')
+        class TestSqlite(test.TestCase):
+            ...
+
     :param connection_name: name of the connection to retrieve capabilities from.
     :param conditions: capability tests which must all pass for the test to run.
     """
 
     def decorator(test_item):
-        @wraps(test_item)
-        def skip_wrapper(*args, **kwargs):
-            db = Tortoise.get_connection(connection_name)
-            for key, val in conditions.items():
-                if getattr(db.capabilities, key) != val:
-                    raise SkipTest("Capability {key} != {val}".format(key=key, val=val))
-            return test_item(*args, **kwargs)
+        if not isinstance(test_item, type):
 
-        return skip_wrapper
+            @wraps(test_item)
+            def skip_wrapper(*args, **kwargs):
+                db = Tortoise.get_connection(connection_name)
+                for key, val in conditions.items():
+                    if getattr(db.capabilities, key) != val:
+                        raise SkipTest(f"Capability {key} != {val}")
+                return test_item(*args, **kwargs)
+
+            return skip_wrapper
+
+        # Assume a class is decorated
+        funcs = {
+            var: getattr(test_item, var)
+            for var in dir(test_item)
+            if var.startswith("test_") and callable(getattr(test_item, var))
+        }
+        for name, func in funcs.items():
+            setattr(
+                test_item,
+                name,
+                requireCapability(connection_name=connection_name, **conditions)(func),
+            )
+
+        return test_item
 
     return decorator
