@@ -1,3 +1,5 @@
+import inspect
+import re
 from copy import copy, deepcopy
 from functools import partial
 from typing import Any, Awaitable, Dict, Generator, List, Optional, Set, Tuple, Type, TypeVar
@@ -115,6 +117,37 @@ def _m2m_getter(
     return val
 
 
+def _get_comments(cls: "Type[Model]") -> Dict[str, str]:
+    """
+    Get comments exactly before attributes
+
+    It can be multiline comment. The placeholder "{model}" will be replaced with the name of the
+    model class. We require that the comments are in #: (with a colon) format, so you can
+    differentiate between private and public comments.
+
+    :param cls: The class we need to extract comments from its source.
+    :return: The dictionary of comments by field name
+    """
+    try:
+        source = inspect.getsource(cls)
+    except TypeError:  # pragma: nocoverage
+        return {}
+    comments = {}
+
+    for cls in reversed(cls.__mro__):
+        if cls is object:
+            continue
+        matches = re.findall(rf"((?:(?!\n|^)[^\w\n]*#:.*?\n)+?)[^\w\n]*(\w+)\s*[:=]", source)
+        for match in matches:
+            field_name = match[1]
+            # Extract text
+            comment = re.sub(r"(^\s*#:\s*|\s*$)", "", match[0], flags=re.MULTILINE)
+            # Class name template
+            comments[field_name] = comment.replace("{model}", cls.__name__)
+
+    return comments
+
+
 class MetaInfo:
     __slots__ = (
         "abstract",
@@ -221,7 +254,7 @@ class MetaInfo:
     @property
     def ordering(self) -> Tuple[Tuple[str, str], ...]:
         if not self._ordering_validated:
-            unknown_fields = set(f for f, _ in self._default_ordering) - self.fields
+            unknown_fields = {f for f, _ in self._default_ordering} - self.fields
             raise ConfigurationError(
                 f"Unknown fields {','.join(unknown_fields)} in "
                 f"default ordering for model {self._model.__name__}"
@@ -548,6 +581,15 @@ class ModelMeta(type):
         for field in meta.fields_map.values():
             field.model = new_class
 
+        for fname, comment in _get_comments(new_class).items():
+            if fname in fields_map:
+                fields_map[fname].docstring = comment
+                if fields_map[fname].description is None:
+                    fields_map[fname].description = comment.split("\n")[0]
+
+        if new_class.__doc__ and not meta.table_description:
+            meta.table_description = inspect.cleandoc(new_class.__doc__).split("\n")[0]
+
         meta._model = new_class
         meta.finalise_fields()
         return new_class
@@ -885,7 +927,11 @@ class Model(metaclass=ModelMeta):
 
     @classmethod
     def _check_together(cls, together: str) -> None:
-        """Check the value of "unique_together" option."""
+        """
+        Check the value of "unique_together" option.
+
+        :raises ConfigurationError: If the model has not been configured correctly.
+        """
         _together = getattr(cls._meta, together)
         if not isinstance(_together, (tuple, list)):
             raise ConfigurationError(f"'{cls.__name__}.{together}' must be a list or tuple.")
