@@ -48,7 +48,9 @@ def _get_joins_for_related_field(
 ) -> List[Tuple[Table, Criterion]]:
     required_joins = []
 
-    related_table: Table = related_field.model_class._meta.basetable
+    related_table = (
+        related_field.model_class._meta.basetable
+    )  # .as_(f"{table.get_table_name()}__{related_field_name}")
     if isinstance(related_field, ManyToManyFieldInstance):
         through_table = Table(related_field.through)
         required_joins.append(
@@ -66,8 +68,6 @@ def _get_joins_for_related_field(
             )
         )
     elif isinstance(related_field, BackwardFKRelation):
-        if table == related_table:
-            related_table = related_table.as_(f"{table.get_table_name()}__{related_field_name}")
         required_joins.append(
             (
                 related_table,
@@ -215,6 +215,8 @@ class Q:
     def __and__(self, other: "Q") -> "Q":
         """
         Returns a binary AND of Q objects, use ``AND`` operator.
+
+        :raises OperationalError: AND operation requires a Q node
         """
         if not isinstance(other, Q):
             raise OperationalError("AND operation requires a Q node")
@@ -223,6 +225,8 @@ class Q:
     def __or__(self, other: "Q") -> "Q":
         """
         Returns a binary OR of Q objects, use ``OR`` operator.
+
+        :raises OperationalError: OR operation requires a Q node
         """
         if not isinstance(other, Q):
             raise OperationalError("OR operation requires a Q node")
@@ -310,7 +314,7 @@ class Q:
             filter_value = value
         else:
             allowed = sorted(
-                list(model._meta.fields | model._meta.fetch_fields | set(self._custom_filters))
+                model._meta.fields | model._meta.fetch_fields | set(self._custom_filters)
             )
             raise FieldError(f"Unknown filter param '{key}'. Allowed base values are {allowed}")
         return filter_key, filter_value
@@ -389,6 +393,7 @@ class Prefetch:
         Called internally to generate prefetching query.
 
         :param queryset: Custom QuerySet to use for prefetching.
+        :raises OperationalError: If field does not exist in model.
         """
         relation_split = self.relation.split("__")
         first_level_field = relation_split[0]
